module Fedi.Routes.Inbox where

import Control.Logger.Simple qualified as Log
import Control.Monad.Catch (SomeException, catch, displayException)
import Fedi.Helpers
import Fedi.Signature.Check
import Fedi.Types
import Fedi.UserDetails
import Web.Twain qualified as Twain
import Web.Twain.Types qualified as Twain
import Prelude hiding (error)

-- * Inbox

matchInbox :: UserDetails -> Twain.PathPattern
matchInbox details =
  fromString ("/" <> details.username <> "/inbox")

handleInbox :: (AnyActivity -> Twain.ResponderM Twain.Response) -> Twain.ResponderM b
handleInbox handle = do
  let
    handler = do
      activity <- checkSignatureAndParseBody
      response <- handle activity
      Twain.send response
  handler `catch` \(e :: SomeException) -> do
    Log.logError (pShow $ displayException e)
    Twain.send $
      -- not necessarily 500, but we'll do this for now.
      Twain.status Twain.status500 $
        Twain.text "Internal Server Error 500"
