module Fedi.UserDetails (
  module Fedi.UserDetails,
) where

import Data.Aeson qualified as A
import Data.Text qualified as T
import Data.Text.Encoding qualified as T
import Fedi.Helpers

type Url = String

type Domain = String

type Username = String

newtype Pem = Pem T.Text
  deriving (Show, Eq)
  deriving (A.FromJSON) via T.Text

pemToBS :: Pem -> ByteString
pemToBS (Pem txt) = T.encodeUtf8 txt

instance A.ToJSON Pem where
  toJSON (Pem pem) = A.String pem

data UserDetails
  = UserDetails
  { domain :: Domain
  , username :: String
  , name :: String
  , summary :: String
  , icon :: Url
  , image :: Url
  , publicPem :: Pem
  , privatePem :: FilePath
  }
  deriving (Show, Eq, Generic, A.FromJSON)

actorUrl :: UserDetails -> Url
actorUrl details =
  "https://" <> details.domain <> "/" <> details.username

fullmention :: UserDetails -> String
fullmention details = "@" <> details.username <> "@" <> details.domain

readUserDetailsFile :: FilePath -> IO UserDetails
readUserDetailsFile detailsFile =
  A.eitherDecodeFileStrict detailsFile
    <&> either (\err -> error $ "could not read file '" <> detailsFile <> "'.\n" <> err) id
