module Fedi.Webfinger where

import Data.Aeson qualified as A
import Fedi.Helpers
import Fedi.UserDetails

data Webfinger
  = Webfinger
  { subject :: Subject
  , links :: [WfLink]
  }
  deriving (Show)

data Subject
  = Subject
  { username :: Username
  , domain :: Domain
  }
  deriving (Show)

ppSubject :: Subject -> String
ppSubject subject =
  "acct:" <> subject.username <> "@" <> subject.domain

data WfLink
  = WfLink
  { type_ :: WfLinkType
  , href :: Url
  }
  deriving (Show)

data WfLinkType = ActivityJson
  deriving (Show)

instance A.ToJSON WfLinkType where
  toJSON ActivityJson = A.String "application/activity+json"

makeWebfinger :: UserDetails -> Webfinger
makeWebfinger details =
  let
    url = "https://" <> details.domain
  in
    Webfinger
      { subject =
          Subject
            { username = details.username
            , domain = details.domain
            }
      , links =
          [ WfLink
              { type_ = ActivityJson
              , href = url <> "/" <> details.username
              }
          ]
      }

-- * -------------------------

---

instance A.ToJSON Webfinger where
  toJSON webfinger =
    A.object
      [ "subject" A..= webfinger.subject
      , "links" A..= webfinger.links
      ]

instance A.ToJSON Subject where
  toJSON subject =
    fromString $ ppSubject subject

instance A.ToJSON WfLink where
  toJSON link =
    A.object
      [ "rel" A..= ("self" :: String)
      , "type" A..= link.type_
      , "href" A..= link.href
      ]
